@@ -34,6 +34,9 @@
 
 #include <errno.h>
 #include <sys/sysinfo.h>
+#include <xf86drm.h>
+#include <amdgpu.h>
+#include <amdgpu_drm.h>
 
 #include "libhsakmt.h"
 #include "fmm.h"
@@ -84,157 +87,212 @@ static const char *supported_processor_vendor_name[] = {
 static HSAKMT_STATUS topology_take_snapshot(void);
 static HSAKMT_STATUS topology_drop_snapshot(void);
 
-int force_asic;
-char force_asic_name[HSA_PUBLIC_NAME_SIZE];
-struct hsa_gfxip_table force_asic_entry = {
-	.amd_name = force_asic_name,
-};
-
 static const struct hsa_gfxip_table gfxip_lookup_table[] = {
 	/* Kaveri Family */
-	{ 0x1304, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1305, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1306, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1307, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1309, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x130A, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x130B, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x130C, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x130D, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x130E, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x130F, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1310, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1311, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1312, 7, 0, 0, 0, "Spooky", CHIP_KAVERI },
-	{ 0x1313, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1315, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x1316, 7, 0, 0, 0, "Spooky", CHIP_KAVERI },
-	{ 0x1317, 7, 0, 0, 0, "Spooky", CHIP_KAVERI },
-	{ 0x1318, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x131B, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x131C, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
-	{ 0x131D, 7, 0, 0, 0, "Spectre", CHIP_KAVERI },
+	{ 0x1304, 7, 0, 0, "Spectre" },
+	{ 0x1305, 7, 0, 0, "Spectre" },
+	{ 0x1306, 7, 0, 0, "Spectre" },
+	{ 0x1307, 7, 0, 0, "Spectre" },
+	{ 0x1309, 7, 0, 0, "Spectre" },
+	{ 0x130A, 7, 0, 0, "Spectre" },
+	{ 0x130B, 7, 0, 0, "Spectre" },
+	{ 0x130C, 7, 0, 0, "Spectre" },
+	{ 0x130D, 7, 0, 0, "Spectre" },
+	{ 0x130E, 7, 0, 0, "Spectre" },
+	{ 0x130F, 7, 0, 0, "Spectre" },
+	{ 0x1310, 7, 0, 0, "Spectre" },
+	{ 0x1311, 7, 0, 0, "Spectre" },
+	{ 0x1312, 7, 0, 0, "Spooky" },
+	{ 0x1313, 7, 0, 0, "Spectre" },
+	{ 0x1315, 7, 0, 0, "Spectre" },
+	{ 0x1316, 7, 0, 0, "Spooky" },
+	{ 0x1317, 7, 0, 0, "Spooky" },
+	{ 0x1318, 7, 0, 0, "Spectre" },
+	{ 0x131B, 7, 0, 0, "Spectre" },
+	{ 0x131C, 7, 0, 0, "Spectre" },
+	{ 0x131D, 7, 0, 0, "Spectre" },
 	/* Hawaii Family */
-	{ 0x67A0, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67A1, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67A2, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67A8, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67A9, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67AA, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67B0, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67B1, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67B8, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67B9, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67BA, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
-	{ 0x67BE, 7, 0, 1, 1, "Hawaii", CHIP_HAWAII },
+	{ 0x67A0, 7, 0, 1, "Hawaii" },
+	{ 0x67A1, 7, 0, 1, "Hawaii" },
+	{ 0x67A2, 7, 0, 1, "Hawaii" },
+	{ 0x67A8, 7, 0, 1, "Hawaii" },
+	{ 0x67A9, 7, 0, 1, "Hawaii" },
+	{ 0x67AA, 7, 0, 1, "Hawaii" },
+	{ 0x67B0, 7, 0, 1, "Hawaii" },
+	{ 0x67B1, 7, 0, 1, "Hawaii" },
+	{ 0x67B8, 7, 0, 1, "Hawaii" },
+	{ 0x67B9, 7, 0, 1, "Hawaii" },
+	{ 0x67BA, 7, 0, 1, "Hawaii" },
+	{ 0x67BE, 7, 0, 1, "Hawaii" },
 	/* Carrizo Family */
-	{ 0x9870, 8, 0, 1, 0, "Carrizo", CHIP_CARRIZO },
-	{ 0x9874, 8, 0, 1, 0, "Carrizo", CHIP_CARRIZO },
-	{ 0x9875, 8, 0, 1, 0, "Carrizo", CHIP_CARRIZO },
-	{ 0x9876, 8, 0, 1, 0, "Carrizo", CHIP_CARRIZO },
-	{ 0x9877, 8, 0, 1, 0, "Carrizo", CHIP_CARRIZO },
+	{ 0x9870, 8, 0, 1, "Carrizo" },
+	{ 0x9874, 8, 0, 1, "Carrizo" },
+	{ 0x9875, 8, 0, 1, "Carrizo" },
+	{ 0x9876, 8, 0, 1, "Carrizo" },
+	{ 0x9877, 8, 0, 1, "Carrizo" },
 	/* Tonga Family */
-	{ 0x6920, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
-	{ 0x6921, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
-	{ 0x6928, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
-	{ 0x6929, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
-	{ 0x692B, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
-	{ 0x692F, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
-	{ 0x6930, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
-	{ 0x6938, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
-	{ 0x6939, 8, 0, 2, 1, "Tonga", CHIP_TONGA },
+	{ 0x6920, 8, 0, 2, "Tonga" },
+	{ 0x6921, 8, 0, 2, "Tonga" },
+	{ 0x6928, 8, 0, 2, "Tonga" },
+	{ 0x6929, 8, 0, 2, "Tonga" },
+	{ 0x692B, 8, 0, 2, "Tonga" },
+	{ 0x692F, 8, 0, 2, "Tonga" },
+	{ 0x6930, 8, 0, 2, "Tonga" },
+	{ 0x6938, 8, 0, 2, "Tonga" },
+	{ 0x6939, 8, 0, 2, "Tonga" },
 	/* Fiji */
-	{ 0x7300, 8, 0, 3, 1, "Fiji", CHIP_FIJI },
-	{ 0x730F, 8, 0, 3, 1, "Fiji", CHIP_FIJI },
+	{ 0x7300, 8, 0, 3, "Fiji" },
+	{ 0x730F, 8, 0, 3, "Fiji" },
 	/* Polaris10 */
-	{ 0x67C0, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67C1, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67C2, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67C4, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67C7, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67C8, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67C9, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67CA, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67CC, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67CF, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67D0, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x67DF, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
-	{ 0x6FDF, 8, 0, 3, 1, "Polaris10", CHIP_POLARIS10 },
+	{ 0x67C0, 8, 0, 3, "Polaris10" },
+	{ 0x67C1, 8, 0, 3, "Polaris10" },
+	{ 0x67C2, 8, 0, 3, "Polaris10" },
+	{ 0x67C4, 8, 0, 3, "Polaris10" },
+	{ 0x67C7, 8, 0, 3, "Polaris10" },
+	{ 0x67C8, 8, 0, 3, "Polaris10" },
+	{ 0x67C9, 8, 0, 3, "Polaris10" },
+	{ 0x67CA, 8, 0, 3, "Polaris10" },
+	{ 0x67CC, 8, 0, 3, "Polaris10" },
+	{ 0x67CF, 8, 0, 3, "Polaris10" },
+	{ 0x67D0, 8, 0, 3, "Polaris10" },
+	{ 0x67DF, 8, 0, 3, "Polaris10" },
+	{ 0x6FDF, 8, 0, 3, "Polaris10" },
 	/* Polaris11 */
-	{ 0x67E0, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
-	{ 0x67E1, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
-	{ 0x67E3, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
-	{ 0x67E7, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
-	{ 0x67E8, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
-	{ 0x67E9, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
-	{ 0x67EB, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
-	{ 0x67EF, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
-	{ 0x67FF, 8, 0, 3, 1, "Polaris11", CHIP_POLARIS11 },
+	{ 0x67E0, 8, 0, 3, "Polaris11" },
+	{ 0x67E1, 8, 0, 3, "Polaris11" },
+	{ 0x67E3, 8, 0, 3, "Polaris11" },
+	{ 0x67E7, 8, 0, 3, "Polaris11" },
+	{ 0x67E8, 8, 0, 3, "Polaris11" },
+	{ 0x67E9, 8, 0, 3, "Polaris11" },
+	{ 0x67EB, 8, 0, 3, "Polaris11" },
+	{ 0x67EF, 8, 0, 3, "Polaris11" },
+	{ 0x67FF, 8, 0, 3, "Polaris11" },
 	/* Polaris12 */
-	{ 0x6980, 8, 0, 3, 1, "Polaris12", CHIP_POLARIS12 },
-	{ 0x6981, 8, 0, 3, 1, "Polaris12", CHIP_POLARIS12 },
-	{ 0x6985, 8, 0, 3, 1, "Polaris12", CHIP_POLARIS12 },
-	{ 0x6986, 8, 0, 3, 1, "Polaris12", CHIP_POLARIS12 },
-	{ 0x6987, 8, 0, 3, 1, "Polaris12", CHIP_POLARIS12 },
-	{ 0x6995, 8, 0, 3, 1, "Polaris12", CHIP_POLARIS12 },
-	{ 0x6997, 8, 0, 3, 1, "Polaris12", CHIP_POLARIS12 },
-	{ 0x699F, 8, 0, 3, 1, "Polaris12", CHIP_POLARIS12 },
+	{ 0x6980, 8, 0, 3, "Polaris12" },
+	{ 0x6981, 8, 0, 3, "Polaris12" },
+	{ 0x6985, 8, 0, 3, "Polaris12" },
+	{ 0x6986, 8, 0, 3, "Polaris12" },
+	{ 0x6987, 8, 0, 3, "Polaris12" },
+	{ 0x6995, 8, 0, 3, "Polaris12" },
+	{ 0x6997, 8, 0, 3, "Polaris12" },
+	{ 0x699F, 8, 0, 3, "Polaris12" },
 	/* VegaM */
-	{ 0x694C, 8, 0, 3, 1, "VegaM", CHIP_VEGAM },
-	{ 0x694E, 8, 0, 3, 1, "VegaM", CHIP_VEGAM },
-	{ 0x694F, 8, 0, 3, 1, "VegaM", CHIP_VEGAM },
+	{ 0x694C, 8, 0, 3, "VegaM" },
+	{ 0x694E, 8, 0, 3, "VegaM" },
+	{ 0x694F, 8, 0, 3, "VegaM" },
 	/* Vega10 */
-	{ 0x6860, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x6861, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x6862, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x6863, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x6864, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x6867, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x6868, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x6869, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x686A, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x686B, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x686C, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x686D, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x686E, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
-	{ 0x687F, 9, 0, 0, 1, "Vega10", CHIP_VEGA10 },
+	{ 0x6860, 9, 0, 0, "Vega10" },
+	{ 0x6861, 9, 0, 0, "Vega10" },
+	{ 0x6862, 9, 0, 0, "Vega10" },
+	{ 0x6863, 9, 0, 0, "Vega10" },
+	{ 0x6864, 9, 0, 0, "Vega10" },
+	{ 0x6867, 9, 0, 0, "Vega10" },
+	{ 0x6868, 9, 0, 0, "Vega10" },
+	{ 0x6869, 9, 0, 0, "Vega10" },
+	{ 0x686A, 9, 0, 0, "Vega10" },
+	{ 0x686B, 9, 0, 0, "Vega10" },
+	{ 0x686C, 9, 0, 0, "Vega10" },
+	{ 0x686D, 9, 0, 0, "Vega10" },
+	{ 0x686E, 9, 0, 0, "Vega10" },
+	{ 0x687F, 9, 0, 0, "Vega10" },
 	/* Vega12 */
-	{ 0x69A0, 9, 0, 4, 1, "Vega12", CHIP_VEGA12 },
-	{ 0x69A1, 9, 0, 4, 1, "Vega12", CHIP_VEGA12 },
-	{ 0x69A2, 9, 0, 4, 1, "Vega12", CHIP_VEGA12 },
-	{ 0x69A3, 9, 0, 4, 1, "Vega12", CHIP_VEGA12 },
-	{ 0x69Af, 9, 0, 4, 1, "Vega12", CHIP_VEGA12 },
+	{ 0x69A0, 9, 0, 4, "Vega12" },
+	{ 0x69A1, 9, 0, 4, "Vega12" },
+	{ 0x69A2, 9, 0, 4, "Vega12" },
+	{ 0x69A3, 9, 0, 4, "Vega12" },
+	{ 0x69Af, 9, 0, 4, "Vega12" },
 	/* Raven */
-	{ 0x15DD, 9, 0, 2, 0, "Raven", CHIP_RAVEN },
-	{ 0x15D8, 9, 0, 2, 0, "Raven", CHIP_RAVEN },
-	/* Renoir */
-	{ 0x1636, 9, 0, 0, 1, "Renoir", CHIP_RENOIR },
+	{ 0x15DD, 9, 0, 2, "Raven" },
+	{ 0x15D8, 9, 0, 2, "Raven" },
 	/* Vega20 */
-	{ 0x66A0, 9, 0, 6, 1, "Vega20", CHIP_VEGA20 },
-	{ 0x66A1, 9, 0, 6, 1, "Vega20", CHIP_VEGA20 },
-	{ 0x66A2, 9, 0, 6, 1, "Vega20", CHIP_VEGA20 },
-	{ 0x66A3, 9, 0, 6, 1, "Vega20", CHIP_VEGA20 },
-	{ 0x66A4, 9, 0, 6, 1, "Vega20", CHIP_VEGA20 },
-	{ 0x66A7, 9, 0, 6, 1, "Vega20", CHIP_VEGA20 },
-	{ 0x66AF, 9, 0, 6, 1, "Vega20", CHIP_VEGA20 },
+	{ 0x66A0, 9, 0, 6, "Vega20" },
+	{ 0x66A1, 9, 0, 6, "Vega20" },
+	{ 0x66A2, 9, 0, 6, "Vega20" },
+	{ 0x66A3, 9, 0, 6, "Vega20" },
+	{ 0x66A4, 9, 0, 6, "Vega20" },
+	{ 0x66A7, 9, 0, 6, "Vega20" },
+	{ 0x66AF, 9, 0, 6, "Vega20" },
 	/* Arcturus */
-	{ 0x7388, 9, 0, 8, 1, "Arcturus", CHIP_ARCTURUS },
-	{ 0x738C, 9, 0, 8, 1, "Arcturus", CHIP_ARCTURUS },
-	{ 0x738E, 9, 0, 8, 1, "Arcturus", CHIP_ARCTURUS },
-	{ 0x7390, 9, 0, 8, 1, "Arcturus", CHIP_ARCTURUS },
+	{ 0x7388, 9, 0, 8, "Arcturus" },
+	{ 0x738C, 9, 0, 8, "Arcturus" },
+	{ 0x738E, 9, 0, 8, "Arcturus" },
+	{ 0x7390, 9, 0, 8, "Arcturus" },
+	/* Aldebaran */
+	{ 0x7408, 9, 0, 10, "Aldebaran" },
+	{ 0x740C, 9, 0, 10, "Aldebaran" },
+	{ 0x740F, 9, 0, 10, "Aldebaran" },
+	{ 0x7410, 9, 0, 10, "Aldebaran" },
+	/* Renoir */
+	{ 0x15E7, 9, 0, 12, "Renoir" },
+	{ 0x1636, 9, 0, 12, "Renoir" },
+	{ 0x1638, 9, 0, 12, "Renoir" },
+	{ 0x164C, 9, 0, 12, "Renoir" },
 	/* Navi10 */
-	{ 0x7310, 10, 1, 0, 1, "Navi10", CHIP_NAVI10 },
-	{ 0x7312, 10, 1, 0, 1, "Navi10", CHIP_NAVI10 },
-	{ 0x7318, 10, 1, 0, 1, "Navi10", CHIP_NAVI10 },
-	{ 0x731A, 10, 1, 0, 1, "Navi10", CHIP_NAVI10 },
-	{ 0x731F, 10, 1, 0, 1, "Navi10", CHIP_NAVI10 },
+	{ 0x7310, 10, 1, 0, "Navi10" },
+	{ 0x7312, 10, 1, 0, "Navi10" },
+	{ 0x7318, 10, 1, 0, "Navi10" },
+	{ 0x731A, 10, 1, 0, "Navi10" },
+	{ 0x731E, 10, 1, 0, "Navi10" },
+	{ 0x731F, 10, 1, 0, "Navi10" },
+	/* cyan_skillfish */
+	{ 0x13F9, 10, 1, 3, "cyan_skillfish" },
+	{ 0x13FA, 10, 1, 3, "cyan_skillfish" },
+	{ 0x13FB, 10, 1, 3, "cyan_skillfish" },
+	{ 0x13FC, 10, 1, 3, "cyan_skillfish" },
+	{ 0x13FE, 10, 1, 3, "cyan_skillfish" },
 	/* Navi14 */
-	{ 0x7340, 10, 1, 2, 1, "Navi14", CHIP_NAVI14 },
-	{ 0x7341, 10, 1, 2, 1, "Navi14", CHIP_NAVI14 },
-	{ 0x7347, 10, 1, 2, 1, "Navi14", CHIP_NAVI14 },
+	{ 0x7340, 10, 1, 2, "Navi14" },
+	{ 0x7341, 10, 1, 2, "Navi14" },
+	{ 0x7347, 10, 1, 2, "Navi14" },
 	/* Navi12 */
-	{ 0x7360, 10, 1, 1, 1, "Navi12", CHIP_NAVI12 },
-	{ 0x7362, 10, 1, 1, 1, "Navi12", CHIP_NAVI12 },
+	{ 0x7360, 10, 1, 1, "Navi12" },
+	{ 0x7362, 10, 1, 1, "Navi12" },
+	/* SIENNA_CICHLID */
+	{ 0x73A0, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73A1, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73A2, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73A3, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73A5, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73A8, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73A9, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73AC, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73AD, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73AB, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73AE, 10, 3, 0, "SIENNA_CICHLID" },
+	{ 0x73BF, 10, 3, 0, "SIENNA_CICHLID" },
+	/* NAVY_FLOUNDER */
+	{ 0x73C0, 10, 3, 1, "NAVY_FLOUNDER" },
+	{ 0x73C1, 10, 3, 1, "NAVY_FLOUNDER" },
+	{ 0x73C3, 10, 3, 1, "NAVY_FLOUNDER" },
+	{ 0x73DA, 10, 3, 1, "NAVY_FLOUNDER" },
+	{ 0x73DB, 10, 3, 1, "NAVY_FLOUNDER" },
+	{ 0x73DC, 10, 3, 1, "NAVY_FLOUNDER" },
+	{ 0x73DD, 10, 3, 1, "NAVY_FLOUNDER" },
+	{ 0x73DE, 10, 3, 1, "NAVY_FLOUNDER" },
+	{ 0x73DF, 10, 3, 1, "NAVY_FLOUNDER" },
+	/* DIMGREY_CAVEFISH */
+	{ 0x73E0, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73E1, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73E2, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73E8, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73E9, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73EA, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73EB, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73EC, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73ED, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73EF, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	{ 0x73FF, 10, 3, 2, "DIMGREY_CAVEFISH" },
+	/* VanGogh */
+	{ 0x163F, 10, 3, 3, "VanGogh" },
+	/* BEIGE_GOBY */
+	{ 0x7420, 10, 3, 4, "BEIGE_GOBY" },
+	{ 0x7421, 10, 3, 4, "BEIGE_GOBY" },
+	{ 0x7422, 10, 3, 4, "BEIGE_GOBY" },
+	{ 0x7423, 10, 3, 4, "BEIGE_GOBY" },
+	{ 0x743F, 10, 3, 4, "BEIGE_GOBY" },
+	/* Yellow_Carp */
+	{ 0x164D, 10, 3, 5, "YELLOW_CARP" },
+	{ 0x1681, 10, 3, 5, "YELLOW_CARP" },
 };
 
 /* information from /proc/cpuinfo */
@@ -640,7 +698,6 @@ HSAKMT_STATUS topology_sysfs_get_system_props(HsaSystemProperties *props)
 	bool is_node_supported = true;
 	uint32_t num_supported_nodes = 0;
 
-
 	assert(props);
 	fd = fopen(KFD_SYSFS_PATH_SYSTEM_PROPERTIES, "r");
 	if (!fd)
@@ -729,9 +786,6 @@ static const struct hsa_gfxip_table *find_hsa_gfxip_device(uint16_t device_id)
 {
 	uint32_t i, table_size;
 
-	if (force_asic)
-		return &force_asic_entry;
-
 	table_size = sizeof(gfxip_lookup_table)/sizeof(struct hsa_gfxip_table);
 	for (i = 0; i < table_size; i++) {
 		if (gfxip_lookup_table[i].device_id == device_id)
@@ -740,42 +794,19 @@ static const struct hsa_gfxip_table *find_hsa_gfxip_device(uint16_t device_id)
 	return NULL;
 }
 
-HSAKMT_STATUS topology_get_asic_family(uint16_t device_id,
-					enum asic_family_type *asic)
+void topology_setup_is_dgpu_param(HsaNodeProperties *props)
 {
-	const struct hsa_gfxip_table *hsa_gfxip =
-				find_hsa_gfxip_device(device_id);
-
-	if (!hsa_gfxip)
-		return HSAKMT_STATUS_INVALID_PARAMETER;
-
-	*asic = hsa_gfxip->asic_family;
-	return HSAKMT_STATUS_SUCCESS;
-}
-
-bool topology_is_dgpu(uint16_t device_id)
-{
-	const struct hsa_gfxip_table *hsa_gfxip =
-				find_hsa_gfxip_device(device_id);
-
-	if (hsa_gfxip && hsa_gfxip->is_dgpu) {
+	/* if we found a dGPU node, then treat the whole system as dGPU */
+	if (!props->NumCPUCores && props->NumFComputeCores)
 		is_dgpu = true;
-		return true;
-	}
-	is_dgpu = false;
-	return false;
 }
 
-bool topology_is_svm_needed(uint16_t device_id)
+bool topology_is_svm_needed(HSA_ENGINE_ID EngineId)
 {
-	const struct hsa_gfxip_table *hsa_gfxip;
-
-	if (topology_is_dgpu(device_id))
+	if (is_dgpu)
 		return true;
 
-	hsa_gfxip = find_hsa_gfxip_device(device_id);
-
-	if (hsa_gfxip && hsa_gfxip->asic_family >= CHIP_VEGA10)
+	if (HSA_GET_GFX_VERSION_FULL(EngineId.ui32) >= GFX_VERSION_VEGA10)
 		return true;
 
 	return false;
@@ -858,7 +889,7 @@ static HSAKMT_STATUS topology_parse_cpuinfo(struct proc_cpuinfo *cpuinfo,
 			p += 2; /* remove ": " */
 			proc = atoi(p);
 			if (proc >= num_procs) {
-				pr_warn("cpuinfo contains processor %d lager than %u\n",
+				pr_warn("cpuinfo contains processor %d larger than %u\n",
 					proc, num_procs);
 				ret = HSAKMT_STATUS_NO_MEMORY;
 				goto exit;
@@ -880,12 +911,10 @@ static HSAKMT_STATUS topology_parse_cpuinfo(struct proc_cpuinfo *cpuinfo,
 			p = strchr(read_buf, ':');
 			p += 2; /* remove ": " */
 			p_len = strlen(p);
-			if (p_len < HSA_PUBLIC_NAME_SIZE) {
-				/* -1 to remove \n from p */
-				strncpy(cpuinfo[proc].model_name, p, p_len - 1);
-				cpuinfo[proc].model_name[p_len - 1] = '\0';
-			} else
-				strncpy(cpuinfo[proc].model_name, p, HSA_PUBLIC_NAME_SIZE);
+			if (p_len > HSA_PUBLIC_NAME_SIZE)
+				p_len = HSA_PUBLIC_NAME_SIZE;
+			memcpy(cpuinfo[proc].model_name, p, p_len);
+			cpuinfo[proc].model_name[p_len - 1] = '\0';
 			continue;
 		}
 
@@ -908,22 +937,53 @@ exit:
 	return ret;
 }
 
+static int topology_get_marketing_name(int minor, uint16_t *marketing_name)
+{
+	int drm_fd;
+	uint32_t major_version;
+	uint32_t minor_version;
+	amdgpu_device_handle device_handle;
+	const char *name;
+	int i;
+
+	if (marketing_name == NULL)
+		return -1;
+	drm_fd = drmOpenRender(minor);
+	if (drm_fd < 0)
+		return -1;
+	if (amdgpu_device_initialize(drm_fd,
+		&major_version, &minor_version, &device_handle) < 0) {
+		drmClose(drm_fd);
+		return -1;
+	}
+	name = amdgpu_get_marketing_name(device_handle);
+	if (name != NULL) {
+		for (i = 0; name[i] != 0 && i < HSA_PUBLIC_NAME_SIZE - 1; i++)
+			marketing_name[i] = name[i];
+		marketing_name[i] = '\0';
+	}
+	amdgpu_device_deinitialize(device_handle);
+	drmClose(drm_fd);
+	return 0;
+}
+
 HSAKMT_STATUS topology_sysfs_get_node_props(uint32_t node_id,
 					    HsaNodeProperties *props,
 					    uint32_t *gpu_id,
-					    struct pci_ids pacc)
+					    bool *p2p_links,
+					    uint32_t *num_p2pLinks)
 {
 	FILE *fd;
 	char *read_buf, *p, *envvar, dummy;
 	char prop_name[256];
 	char path[256];
 	unsigned long long prop_val;
-	uint32_t i, prog, major, minor, step;
+	uint32_t prog, major, minor, step;
 	int read_size;
 	const struct hsa_gfxip_table *hsa_gfxip;
-	char namebuf[HSA_PUBLIC_NAME_SIZE];
-	const char *name;
 	uint32_t sys_node_id;
+	uint32_t gfxv = 0;
+	uint8_t gfxv_major, gfxv_minor, gfxv_stepping;
 
 	HSAKMT_STATUS ret = HSAKMT_STATUS_SUCCESS;
 
@@ -973,7 +1033,13 @@ HSAKMT_STATUS topology_sysfs_get_node_props(uint32_t node_id,
 			props->NumCaches = (uint32_t)prop_val;
 		else if (strcmp(prop_name, "io_links_count") == 0)
 			props->NumIOLinks = (uint32_t)prop_val;
-		else if (strcmp(prop_name, "cpu_core_id_base") == 0)
+		else if (strcmp(prop_name, "p2p_links_count") == 0) {
+			props->NumIOLinks += (uint32_t)prop_val;
+			if (num_p2pLinks)
+				*num_p2pLinks = (uint32_t)prop_val;
+			if (p2p_links)
+				*p2p_links = true;
+		} else if (strcmp(prop_name, "cpu_core_id_base") == 0)
 			props->CComputeIdLo = (uint32_t)prop_val;
 		else if (strcmp(prop_name, "simd_id_base") == 0)
 			props->FComputeIdLo = (uint32_t)prop_val;
@@ -1033,10 +1099,16 @@ HSAKMT_STATUS topology_sysfs_get_node_props(uint32_t node_id,
 			props->NumSdmaQueuesPerEngine = prop_val;
 		else if (strcmp(prop_name, "num_cp_queues") == 0)
 			props->NumCpQueues = prop_val;
+		else if (strcmp(prop_name, "gfx_target_version") == 0)
+			gfxv = (uint32_t)prop_val;
 	}
 
+	gfxv_major = HSA_GET_GFX_VERSION_MAJOR(gfxv);
+	gfxv_minor = HSA_GET_GFX_VERSION_MINOR(gfxv);
+	gfxv_stepping = HSA_GET_GFX_VERSION_STEP(gfxv);
+
 	hsa_gfxip = find_hsa_gfxip_device(props->DeviceId);
-	if (hsa_gfxip) {
+	if (hsa_gfxip || gfxv) {
 		envvar = getenv("HSA_OVERRIDE_GFX_VERSION");
 		if (envvar) {
 			/* HSA_OVERRIDE_GFX_VERSION=major.minor.stepping */
@@ -1051,37 +1123,47 @@ HSAKMT_STATUS topology_sysfs_get_node_props(uint32_t node_id,
 			props->EngineId.ui32.Major = major & 0x3f;
 			props->EngineId.ui32.Minor = minor & 0xff;
 			props->EngineId.ui32.Stepping = step & 0xff;
-		} else {
+		} else if (hsa_gfxip) {
 			props->EngineId.ui32.Major = hsa_gfxip->major & 0x3f;
-			props->EngineId.ui32.Minor = hsa_gfxip->minor;
-			props->EngineId.ui32.Stepping = hsa_gfxip->stepping;
+			props->EngineId.ui32.Minor = hsa_gfxip->minor & 0xff;
+			props->EngineId.ui32.Stepping = hsa_gfxip->stepping & 0xff;
+		} else {
+			props->EngineId.ui32.Major = gfxv_major & 0x3f;
+			props->EngineId.ui32.Minor = gfxv_minor & 0xff;
+			props->EngineId.ui32.Stepping = gfxv_stepping & 0xff;
 		}
 
-		if (!hsa_gfxip->amd_name) {
-			ret = HSAKMT_STATUS_ERROR;
-			goto err;
-		}
+		/* Set the CAL name of the node. If DID-based hsa_gfxip lookup was
+		 * successful, use that name. Otherwise, set to GFX<GFX_VERSION>.
+		 */
+		if (hsa_gfxip && hsa_gfxip->amd_name)
+			strncpy((char *)props->AMDName, hsa_gfxip->amd_name,
+					sizeof(props->AMDName)-1);
+		else
+			snprintf((char *)props->AMDName, sizeof(props->AMDName)-1, "GFX%06x",
+					HSA_GET_GFX_VERSION_FULL(props->EngineId.ui32));
 
-		/* Retrieve the CAL name of the node */
-		strncpy((char *)props->AMDName, hsa_gfxip->amd_name, sizeof(props->AMDName)-1);
 		if (!props->NumCPUCores) {
 			/* Is dGPU Node, not APU
-			 * Retrieve the marketing name of the node using pcilib,
-			 * convert UTF8 to UTF16
+			 * Retrieve the marketing name of the node.
 			 */
-			name = pci_ids_lookup(pacc, namebuf, sizeof(namebuf),
-								   props->VendorId, props->DeviceId);
-			for (i = 0; name[i] != 0 && i < HSA_PUBLIC_NAME_SIZE - 1; i++)
-				props->MarketingName[i] = name[i];
-			props->MarketingName[i] = '\0';
+			if (topology_get_marketing_name(props->DrmRenderMinor,
+					props->MarketingName) != 0)
+				pr_info("failed to get marketing name for device ID 0x%x\n",
+						props->DeviceId);
 		}
+
+		/* Get VGPR/SGPR size in byte per CU */
+		props->SGPRSizePerCU = SGPR_SIZE_PER_CU;
+		props->VGPRSizePerCU = VGPR_SIZE_PER_CU(HSA_GET_GFX_VERSION_FULL(props->EngineId.ui32));
+
 	} else if (props->DeviceId)
 		/* still return success */
 		pr_err("device ID 0x%x is not supported in libhsakmt\n",
 				props->DeviceId);
 
 	if (props->NumFComputeCores)
-		assert(props->EngineId.ui32.Major);
+		assert(props->EngineId.ui32.Major && "HSA_OVERRIDE_GFX_VERSION may be needed");
 
 err:
 	free(read_buf);
@@ -1410,7 +1492,7 @@ static HSAKMT_STATUS topology_map_sysfs_to_user_node_id(uint32_t sys_node_id, ui
  */
 static HSAKMT_STATUS topology_sysfs_get_iolink_props(uint32_t node_id,
 						     uint32_t iolink_id,
-						     HsaIoLinkProperties *props)
+						     HsaIoLinkProperties *props, bool p2pLink)
 {
 	FILE *fd;
 	char *read_buf, *p;
@@ -1427,7 +1509,11 @@ static HSAKMT_STATUS topology_sysfs_get_iolink_props(uint32_t node_id,
 	if (ret != HSAKMT_STATUS_SUCCESS)
 		return ret;
 
-	snprintf(path, 256, "%s/%d/io_links/%d/properties", KFD_SYSFS_PATH_NODES, sys_node_id, iolink_id);
+	if (p2pLink)
+		snprintf(path, 256, "%s/%d/p2p_links/%d/properties", KFD_SYSFS_PATH_NODES, sys_node_id, iolink_id);
+	else
+		snprintf(path, 256, "%s/%d/io_links/%d/properties", KFD_SYSFS_PATH_NODES, sys_node_id, iolink_id);
+
 	fd = fopen(path, "r");
 	if (!fd)
 		return HSAKMT_STATUS_ERROR;
@@ -1439,7 +1525,8 @@ static HSAKMT_STATUS topology_sysfs_get_iolink_props(uint32_t node_id,
 
 	read_size = fread(read_buf, 1, PAGE_SIZE, fd);
 	if (read_size <= 0) {
-		ret = HSAKMT_STATUS_ERROR;
+		ret = (errno == EPERM) ? HSAKMT_STATUS_NOT_SUPPORTED :
+					 HSAKMT_STATUS_ERROR;
 		goto err2;
 	}
 
@@ -1732,9 +1819,11 @@ HSAKMT_STATUS topology_take_snapshot(void)
 	HsaSystemProperties sys_props;
 	node_props_t *temp_props = 0;
 	HSAKMT_STATUS ret = HSAKMT_STATUS_SUCCESS;
-	struct pci_ids pacc;
 	struct proc_cpuinfo *cpuinfo;
 	const uint32_t num_procs = get_nprocs();
+	uint32_t num_ioLinks;
+	bool p2p_links = false;
+	uint32_t num_p2pLinks = 0;
 
 	cpuinfo = calloc(num_procs, sizeof(struct proc_cpuinfo));
 	if (!cpuinfo) {
@@ -1756,11 +1845,11 @@ retry:
 			ret = HSAKMT_STATUS_NO_MEMORY;
 			goto err;
 		}
-		pacc = pci_ids_create();
 		for (i = 0; i < sys_props.NumNodes; i++) {
 			ret = topology_sysfs_get_node_props(i,
 					&temp_props[i].node,
-					&temp_props[i].gpu_id, pacc);
+					&temp_props[i].gpu_id,
+					&p2p_links, &num_p2pLinks);
 			if (ret != HSAKMT_STATUS_SUCCESS) {
 				free_properties(temp_props, i);
 				goto err;
@@ -1818,17 +1907,19 @@ retry:
 				free_properties(temp_props, i + 1);
 				goto err;
 			}
+			num_ioLinks = temp_props[i].node.NumIOLinks - num_p2pLinks;
+			uint32_t link_id = 0;
 
-			if (temp_props[i].node.NumIOLinks) {
-				uint32_t sys_link_id = 0, link_id = 0;
+			if (num_ioLinks) {
+				uint32_t sys_link_id = 0;
 
 				/* Parse all the sysfs specified io links. Skip the ones where the
 				 * remote node (node_to) is not accessible
 				 */
-				while (sys_link_id < temp_props[i].node.NumIOLinks &&
+				while (sys_link_id < num_ioLinks &&
 					link_id < sys_props.NumNodes - 1) {
 					ret = topology_sysfs_get_iolink_props(i, sys_link_id++,
-									      &temp_props[i].link[link_id]);
+								&temp_props[i].link[link_id], false);
 					if (ret == HSAKMT_STATUS_NOT_SUPPORTED) {
 						ret = HSAKMT_STATUS_SUCCESS;
 						continue;
@@ -1841,14 +1932,36 @@ retry:
 				/* sysfs specifies all the io links. Limit the number to valid ones */
 				temp_props[i].node.NumIOLinks = link_id;
 			}
+
+			if (num_p2pLinks) {
+				uint32_t sys_link_id = 0;
+
+				/* Parse all the sysfs specified p2p links.
+				 */
+				while (sys_link_id < num_p2pLinks &&
+					link_id < sys_props.NumNodes - 1) {
+					ret = topology_sysfs_get_iolink_props(i, sys_link_id++,
+								&temp_props[i].link[link_id], true);
+					if (ret == HSAKMT_STATUS_NOT_SUPPORTED) {
+						ret = HSAKMT_STATUS_SUCCESS;
+						continue;
+					} else if (ret != HSAKMT_STATUS_SUCCESS) {
+						free_properties(temp_props, i + 1);
+						goto err;
+					}
+					link_id++;
+				}
+				temp_props[i].node.NumIOLinks = link_id;
+			}
 		}
-		pci_ids_destroy(pacc);
 	}
 
-	/* All direct IO links are created in the kernel. Here we need to
-	 * connect GPU<->GPU or GPU<->CPU indirect IO links.
-	 */
-	topology_create_indirect_gpu_links(&sys_props, temp_props);
+	if (!p2p_links) {
+		/* All direct IO links are created in the kernel. Here we need to
+		 * connect GPU<->GPU or GPU<->CPU indirect IO links.
+		 */
+		topology_create_indirect_gpu_links(&sys_props, temp_props);
+	}
 
 	ret = topology_sysfs_get_generation(&gen_end);
 	if (ret != HSAKMT_STATUS_SUCCESS) {
@@ -1987,27 +2100,15 @@ HSAKMT_STATUS HSAKMTAPI hsaKmtGetNodeProperties(HSAuint32 NodeId,
 	CHECK_KFD_OPEN();
 	pthread_mutex_lock(&hsakmt_mutex);
 
-	/* KFD ADD page 18, snapshot protocol violation */
-	if (!g_system) {
-		err = HSAKMT_STATUS_INVALID_NODE_UNIT;
-		assert(g_system);
-		goto out;
-	}
-
-	if (NodeId >= g_system->NumNodes) {
-		err = HSAKMT_STATUS_INVALID_PARAMETER;
-		goto out;
-	}
-
 	err = validate_nodeid(NodeId, &gpu_id);
 	if (err != HSAKMT_STATUS_SUCCESS)
-		return err;
+		goto out;
 
 	*NodeProperties = g_props[NodeId].node;
 	/* For CPU only node don't add any additional GPU memory banks. */
 	if (gpu_id) {
 		uint64_t base, limit;
-		if (topology_is_dgpu(get_device_id_by_gpu_id(gpu_id)))
+		if (is_dgpu)
 			NodeProperties->NumMemoryBanks += NUM_OF_DGPU_HEAPS;
 		else
 			NodeProperties->NumMemoryBanks += NUM_OF_IGPU_HEAPS;
@@ -2036,19 +2137,6 @@ HSAKMT_STATUS HSAKMTAPI hsaKmtGetNodeMemoryProperties(HSAuint32 NodeId,
 	CHECK_KFD_OPEN();
 	pthread_mutex_lock(&hsakmt_mutex);
 
-	/* KFD ADD page 18, snapshot protocol violation */
-	if (!g_system) {
-		err = HSAKMT_STATUS_INVALID_NODE_UNIT;
-		assert(g_system);
-		goto out;
-	}
-
-	/* Check still necessary */
-	if (NodeId >= g_system->NumNodes) {
-		err = HSAKMT_STATUS_INVALID_PARAMETER;
-		goto out;
-	}
-
 	err = validate_nodeid(NodeId, &gpu_id);
 	if (err != HSAKMT_STATUS_SUCCESS)
 		goto out;
@@ -2077,7 +2165,8 @@ HSAKMT_STATUS HSAKMTAPI hsaKmtGetNodeMemoryProperties(HSAuint32 NodeId,
 	 * For dGPU the topology node contains Local Memory and it is added by
 	 * the for loop above
 	 */
-	if (is_kaveri(NodeId) && i < NumBanks && g_props[NodeId].node.LocalMemSize > 0 &&
+	if (get_gfxv_by_node_id(NodeId) == GFX_VERSION_KAVERI && i < NumBanks &&
+		g_props[NodeId].node.LocalMemSize > 0 &&
 		fmm_get_aperture_base_and_limit(FMM_GPUVM, gpu_id,
 				&MemoryProperties[i].VirtualBaseAddress, &aperture_limit) == HSAKMT_STATUS_SUCCESS) {
 		MemoryProperties[i].HeapType = HSA_HEAPTYPE_FRAME_BUFFER_PRIVATE;
@@ -2095,7 +2184,7 @@ HSAKMT_STATUS HSAKMTAPI hsaKmtGetNodeMemoryProperties(HSAuint32 NodeId,
 	}
 
 	/* Add SVM aperture */
-	if (topology_is_svm_needed(get_device_id_by_gpu_id(gpu_id)) && i < NumBanks &&
+	if (topology_is_svm_needed(g_props[NodeId].node.EngineId) && i < NumBanks &&
 	    fmm_get_aperture_base_and_limit(
 		    FMM_SVM, gpu_id, &MemoryProperties[i].VirtualBaseAddress,
 		    &aperture_limit) == HSAKMT_STATUS_SUCCESS) {
@@ -2133,13 +2222,12 @@ HSAKMT_STATUS HSAKMTAPI hsaKmtGetNodeCacheProperties(HSAuint32 NodeId,
 	pthread_mutex_lock(&hsakmt_mutex);
 
 	/* KFD ADD page 18, snapshot protocol violation */
-	if (!g_system) {
+	if (!g_system || NodeId >= g_system->NumNodes) {
 		err = HSAKMT_STATUS_INVALID_NODE_UNIT;
-		assert(g_system);
 		goto out;
 	}
 
-	if (NodeId >= g_system->NumNodes || NumCaches > g_props[NodeId].node.NumCaches) {
+	if (NumCaches > g_props[NodeId].node.NumCaches) {
 		err = HSAKMT_STATUS_INVALID_PARAMETER;
 		goto out;
 	}
@@ -2171,13 +2259,12 @@ HSAKMT_STATUS HSAKMTAPI hsaKmtGetNodeIoLinkProperties(HSAuint32 NodeId,
 	pthread_mutex_lock(&hsakmt_mutex);
 
 	/* KFD ADD page 18, snapshot protocol violation */
-	if (!g_system) {
+	if (!g_system || NodeId >= g_system->NumNodes ) {
 		err = HSAKMT_STATUS_INVALID_NODE_UNIT;
-		assert(g_system);
 		goto out;
 	}
 
-	if (NodeId >= g_system->NumNodes || NumIoLinks > g_props[NodeId].node.NumIOLinks) {
+	if (NumIoLinks > g_props[NodeId].node.NumIOLinks) {
 		err = HSAKMT_STATUS_INVALID_PARAMETER;
 		goto out;
 	}
@@ -2194,6 +2281,11 @@ out:
 	return err;
 }
 
+uint32_t get_gfxv_by_node_id(HSAuint32 node_id)
+{
+	return HSA_GET_GFX_VERSION_FULL(g_props[node_id].node.EngineId.ui32);
+}
+
 uint16_t get_device_id_by_node_id(HSAuint32 node_id)
 {
 	if (!g_props || !g_system || g_system->NumNodes <= node_id)
@@ -2209,12 +2301,6 @@ bool prefer_ats(HSAuint32 node_id)
 			&& g_props[node_id].node.NumFComputeCores;
 }
 
-bool is_kaveri(HSAuint32 node_id)
-{
-	return g_props[node_id].node.EngineId.ui32.Major == 7
-			&& g_props[node_id].node.EngineId.ui32.Minor == 0;
-}
-
 uint16_t get_device_id_by_gpu_id(HSAuint32 gpu_id)
 {
 	unsigned int i;
