@@ -213,6 +213,7 @@ void KFDEvictTest::AmdgpuCommandSubmissionSdmaNop(int rn, amdgpu_bo_handle handl
     amdgpu_va_handle vaHandle;
     uint32_t *ptr;
     uint32_t expired;
+    unsigned failCount = 0;
 
     ASSERT_EQ(0, amdgpu_cs_ctx_create(m_RenderNodes[rn].device_handle, &contextHandle));
 
@@ -244,8 +245,14 @@ void KFDEvictTest::AmdgpuCommandSubmissionSdmaNop(int rn, amdgpu_bo_handle handl
 
     memset(&fenceStatus, 0, sizeof(struct amdgpu_cs_fence));
     for (int i = 0; i < 100; i++) {
-        ASSERT_EQ(0, amdgpu_cs_submit(contextHandle, 0, &ibsRequest, 1));
+        int r = amdgpu_cs_submit(contextHandle, 0, &ibsRequest, 1);
+
         Delay(50);
+        if (r) {
+            failCount++;
+            ASSERT_LE(failCount, 2);
+            continue;
+        }
 
         fenceStatus.context = contextHandle;
         fenceStatus.ip_type = AMDGPU_HW_IP_DMA;
@@ -471,7 +478,7 @@ TEST_F(KFDEvictTest, BasicTest) {
     AllocBuffers(defaultGPUNode, count, vramBufSize, pBuffers);
 
     /* Allocate gfx vram size of at most one third system memory */
-    HSAuint64 size = sysMemSize / 3 < testSize ? sysMemSize / 3 : testSize;
+    HSAuint64 size = sysMemSize / 3 < testSize / 2 ? sysMemSize / 3 : testSize / 2;
     amdgpu_bo_handle handle;
     AllocAmdgpuBo(rn, size, handle);
 
@@ -569,7 +576,7 @@ TEST_F(KFDEvictTest, QueueTest) {
     AllocBuffers(defaultGPUNode, count, vramBufSize, pBuffers);
 
     /* Allocate gfx vram size of at most one third system memory */
-    HSAuint64 size = sysMemSize / 3 < testSize ? sysMemSize / 3 : testSize;
+    HSAuint64 size = sysMemSize / 3 < testSize / 2 ? sysMemSize / 3 : testSize / 2;
     amdgpu_bo_handle handle;
     AllocAmdgpuBo(rn, size, handle);
 
@@ -596,7 +603,7 @@ TEST_F(KFDEvictTest, QueueTest) {
     addrBuffer.Fill(0x5678);
 
     /* Wait for shader to finish or timeout if shader has vm page fault */
-    EXPECT_EQ(0, dispatch0.SyncWithStatus(120000));
+    EXPECT_EQ(0, dispatch0.SyncWithStatus(180000));
 
     EXPECT_SUCCESS(pm4Queue.Destroy());
 
@@ -668,7 +675,7 @@ TEST_F(KFDEvictTest, BurstyTest) {
     AllocBuffers(defaultGPUNode, count, vramBufSize, pBuffers);
 
     /* Allocate gfx vram size of at most one third system memory */
-    HSAuint64 size = sysMemSize / 3 < testSize ? sysMemSize / 3 : testSize;
+    HSAuint64 size = sysMemSize / 3 < testSize / 2 ? sysMemSize / 3 : testSize / 2;
     amdgpu_bo_handle handle;
     AllocAmdgpuBo(rn, size, handle);
 
