@@ -22,9 +22,9 @@
  */
 
 #include "KFDTestUtil.hpp"
-
 #include <stdlib.h>
 #include <sys/time.h>
+#include <sys/mman.h>
 #include <algorithm>
 #include <vector>
 #include "BaseQueue.hpp"
@@ -149,8 +149,10 @@ unsigned int FamilyIdFromNode(const HsaNodeProperties *props) {
         familyId = FAMILY_AI;
         if (props->EngineId.ui32.Stepping == 2)
             familyId = FAMILY_RV;
-        if (props->EngineId.ui32.Stepping == 8)
+        else if (props->EngineId.ui32.Stepping == 8)
             familyId = FAMILY_AR;
+        else if (props->EngineId.ui32.Stepping == 10)
+            familyId = FAMILY_AL;
         break;
     case 10:
         familyId = FAMILY_NV;
@@ -201,7 +203,7 @@ HSAuint64 GetSystemTickCountInMicroSec() {
 const HsaMemoryBuffer HsaMemoryBuffer::Null;
 
 HsaMemoryBuffer::HsaMemoryBuffer(HSAuint64 size, unsigned int node, bool zero, bool isLocal, bool isExec,
-                                 bool isScratch, bool isReadOnly)
+                                 bool isScratch, bool isReadOnly, bool isUncached)
     :m_Size(size),
     m_pUser(NULL),
     m_pBuf(NULL),
@@ -222,11 +224,13 @@ HsaMemoryBuffer::HsaMemoryBuffer(HSAuint64 size, unsigned int node, bool zero, b
             m_Flags.ui32.HostAccess = 0;
             m_Flags.ui32.NonPaged = 1;
             m_Flags.ui32.CoarseGrain = 1;
+            EXPECT_EQ(isUncached, 0) << "Uncached flag is relevant only for system or host memory";
         } else {
             m_Flags.ui32.HostAccess = 1;
             m_Flags.ui32.NonPaged = 0;
             m_Flags.ui32.CoarseGrain = 0;
             m_Flags.ui32.NoNUMABind = 1;
+            m_Flags.ui32.Uncached = isUncached;
         }
 
         if (isExec)
@@ -616,6 +620,23 @@ const bool HsaNodeInfo::IsGPUNodeLargeBar(int node) const {
     return false;
 }
 
+const bool HsaNodeInfo::IsPeerAccessibleByNode(int peer, int node) const {
+    const HsaNodeProperties *pNodeProperties;
+
+    pNodeProperties = GetNodeProperties(node);
+    if (pNodeProperties) {
+        HsaIoLinkProperties p2pLinksProperties[pNodeProperties->NumIOLinks];
+        EXPECT_SUCCESS(hsaKmtGetNodeIoLinkProperties(node,
+					pNodeProperties->NumIOLinks, p2pLinksProperties));
+
+        for (unsigned link = 0; link < pNodeProperties->NumIOLinks; link++)
+            if (p2pLinksProperties[link].NodeTo == peer)
+                return true;
+    }
+
+    return false;
+}
+
 const int HsaNodeInfo::FindLargeBarGPUNode() const {
     const std::vector<int> gpuNodes = GetNodesWithGPU();
 
@@ -637,27 +658,217 @@ const bool HsaNodeInfo::AreGPUNodesXGMI(int node0, int node1) const {
     return false;
 }
 
-int HsaNodeInfo::FindAccessiblePeers(std::vector<HSAuint32> *peers, HSAuint32 dstNode,
-        bool bidirectional) const {
-    peers->push_back(dstNode);
-    if (IsGPUNodeLargeBar(dstNode)) {
-        for (unsigned i = 0; i < m_NodesWithGPU.size(); i++) {
-            if (m_NodesWithGPU.at(i) == dstNode)
-                continue;
+int HsaNodeInfo::FindAccessiblePeers(std::vector<int> *peers,
+		                             HSAuint32 node) const {
+    peers->push_back(node);
 
-            if (!bidirectional || IsGPUNodeLargeBar(m_NodesWithGPU.at(i)) ||
-                AreGPUNodesXGMI(dstNode, m_NodesWithGPU.at(i)))
-                peers->push_back(m_NodesWithGPU.at(i));
-        }
-    } else {
-        for (unsigned i = 0; i < m_NodesWithGPU.size(); i++) {
-            if (m_NodesWithGPU.at(i) == dstNode)
-                continue;
+    for (unsigned i = 0; i < m_NodesWithGPU.size(); i++) {
+        if (m_NodesWithGPU.at(i) == node)
+            continue;
 
-            if (AreGPUNodesXGMI(dstNode, m_NodesWithGPU.at(i)))
-                peers->push_back(m_NodesWithGPU.at(i));
-        }
+        if (IsPeerAccessibleByNode(m_NodesWithGPU.at(i), node))
+            peers->push_back(m_NodesWithGPU.at(i));
     }
-
     return peers->size();
 }
+
+const bool HsaNodeInfo::IsNodeXGMItoCPU(int node) const {
+    const HsaNodeProperties *pNodeProperties;
+    bool ret = false;
+
+    pNodeProperties = GetNodeProperties(node);
+    if (pNodeProperties && pNodeProperties->NumIOLinks) {
+        HsaIoLinkProperties  *IolinkProperties =  new HsaIoLinkProperties[pNodeProperties->NumIOLinks];
+        EXPECT_SUCCESS(hsaKmtGetNodeIoLinkProperties(node, pNodeProperties->NumIOLinks, IolinkProperties));
+
+        for (int linkId = 0; linkId < pNodeProperties->NumIOLinks; linkId++) {
+            EXPECT_EQ(node, IolinkProperties[linkId].NodeFrom);
+            const HsaNodeProperties *pNodeProperties0 =
+                    GetNodeProperties(IolinkProperties[linkId].NodeTo);
+            if (pNodeProperties0->NumFComputeCores == 0 &&
+                    IolinkProperties[linkId].IoLinkType == HSA_IOLINK_TYPE_XGMI)
+                ret = true;
+        }
+        delete [] IolinkProperties;
+    }
+
+    return ret;
+}
+
+HSAKMT_STATUS RegisterSVMRange(HSAuint32 GPUNode, void *MemoryAddress,
+                               HSAuint64 SizeInBytes, HSAuint32 PrefetchNode,
+                               HSAuint32 SVMFlags) {
+    HSA_SVM_ATTRIBUTE *attrs;
+    HSAuint64 s_attr;
+    HSAuint32 nattr;
+    HSAKMT_STATUS r;
+
+    nattr = 4;
+    s_attr = sizeof(*attrs) * nattr;
+    attrs = (HSA_SVM_ATTRIBUTE *)alloca(s_attr);
+
+    attrs[0].type = HSA_SVM_ATTR_PREFETCH_LOC;
+    attrs[0].value = PrefetchNode;
+    attrs[1].type = HSA_SVM_ATTR_PREFERRED_LOC;
+    attrs[1].value = PrefetchNode;
+    attrs[2].type = HSA_SVM_ATTR_SET_FLAGS;
+    attrs[2].value = SVMFlags;
+    attrs[3].type = HSA_SVM_ATTR_ACCESS;
+    attrs[3].value = GPUNode;
+
+    r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, nattr, attrs);
+    if (r) {
+        LOG() << "set range attrs failed" << std::endl;
+        return HSAKMT_STATUS_ERROR;
+    }
+
+    return HSAKMT_STATUS_SUCCESS;
+}
+
+HSAKMT_STATUS SVMRangeGetPrefetchNode(void *MemoryAddress, HSAuint64 SizeInBytes,
+                                      HSAuint32 *PrefetchNode) {
+    HSA_SVM_ATTRIBUTE attr;
+    int r;
+
+    attr.type = HSA_SVM_ATTR_PREFETCH_LOC;
+    attr.value = 0;
+
+    r = hsaKmtSVMGetAttr(MemoryAddress, SizeInBytes, 1, &attr);
+    if (r) {
+        LOG() << "get prefetch node failed" << std::endl;
+        return HSAKMT_STATUS_ERROR;
+    }
+
+    *PrefetchNode = attr.value;
+
+    return HSAKMT_STATUS_SUCCESS;
+}
+
+HSAKMT_STATUS SVMRangePrefetchToNode(void *MemoryAddress, HSAuint64 SizeInBytes,
+                                           HSAuint32 PrefetchNode) {
+    HSA_SVM_ATTRIBUTE attr;
+    int r;
+
+    attr.type = HSA_SVM_ATTR_PREFETCH_LOC;
+    attr.value = PrefetchNode;
+
+    r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, 1, &attr);
+    if (r) {
+        LOG() << "set prefetch node failed" << std::endl;
+        return HSAKMT_STATUS_ERROR;
+    }
+
+    return HSAKMT_STATUS_SUCCESS;
+}
+
+HSAKMT_STATUS SVMRangeMapToNode(void *MemoryAddress, HSAuint64 SizeInBytes,
+                                           HSAuint32 NodeID) {
+    HSA_SVM_ATTRIBUTE attr;
+    int r;
+
+    attr.type = HSA_SVM_ATTR_ACCESS;
+    attr.value = NodeID;
+
+    r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, 1, &attr);
+    if (r) {
+        LOG() << "set map to node failed" << std::endl;
+        return HSAKMT_STATUS_ERROR;
+    }
+
+    return HSAKMT_STATUS_SUCCESS;
+}
+
+HSAKMT_STATUS SVMRangeMapInPlaceToNode(void *MemoryAddress, HSAuint64 SizeInBytes,
+                                           HSAuint32 NodeID) {
+    HSA_SVM_ATTRIBUTE attr;
+    int r;
+
+    attr.type = HSA_SVM_ATTR_ACCESS_IN_PLACE;
+    attr.value = NodeID;
+
+    r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, 1, &attr);
+    if (r) {
+        LOG() << "set map in place to node failed" << std::endl;
+        return HSAKMT_STATUS_ERROR;
+    }
+
+    return HSAKMT_STATUS_SUCCESS;
+}
+
+HSAKMT_STATUS SVMRangSetGranularity(void *MemoryAddress, HSAuint64 SizeInBytes,
+                                    HSAuint32 Granularity) {
+    HSA_SVM_ATTRIBUTE attr;
+    int r;
+
+    attr.type = HSA_SVM_ATTR_GRANULARITY;
+    attr.value = Granularity;
+
+    r = hsaKmtSVMSetAttr(MemoryAddress, SizeInBytes, 1, &attr);
+    if (r) {
+        LOG() << "set granularity failed" << std::endl;
+        return HSAKMT_STATUS_ERROR;
+    }
+
+    return HSAKMT_STATUS_SUCCESS;
+}
+
+HsaSVMRange::HsaSVMRange(HSAuint64 size, HSAuint32 GPUNode) :
+    HsaSVMRange(NULL, size, GPUNode, 0) {}
+
+HsaSVMRange::HsaSVMRange(HSAuint64 size) :
+    HsaSVMRange(NULL, size, 0, 0, true) {}
+
+HsaSVMRange::HsaSVMRange(HSAuint64 size, HSAuint32 GPUNode, HSAuint32 PrefetchNode) :
+    HsaSVMRange(NULL, size, GPUNode, PrefetchNode) {}
+
+HsaSVMRange::HsaSVMRange(void *addr, HSAuint64 size, HSAuint32 GPUNode, HSAuint32 PrefetchNode,
+                         bool noRegister, bool isLocal, bool isExec, bool isReadOnly):
+    m_Size(size),
+    m_pUser(addr),
+    m_Local(isLocal),
+    m_Node(PrefetchNode),
+    m_SelfAllocated(false) {
+    if (!m_pUser) {
+        m_pUser = mmap(0, m_Size, PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+        EXPECT_NOTNULL(m_pUser);
+        m_SelfAllocated = true;
+    }
+
+    if (m_Local)
+        m_Flags = HSA_SVM_FLAG_HOST_ACCESS;
+    else
+        m_Flags = HSA_SVM_FLAG_HOST_ACCESS | HSA_SVM_FLAG_COHERENT;
+
+    if (isReadOnly)
+        m_Flags |= HSA_SVM_FLAG_GPU_RO;
+    if (isExec)
+        m_Flags |= HSA_SVM_FLAG_GPU_EXEC;
+
+    if (!noRegister)
+        EXPECT_SUCCESS(RegisterSVMRange(GPUNode, m_pUser, m_Size, PrefetchNode, m_Flags));
+}
+
+HsaSVMRange::~HsaSVMRange() {
+    if (m_pUser != NULL) {
+        if (m_SelfAllocated)
+            munmap(m_pUser, m_Size);
+        m_pUser = NULL;
+    }
+}
+
+void HsaSVMRange::Fill(HSAuint32 value, HSAuint64 offset, HSAuint64 size) {
+    HSAuint64 i;
+    HSAuint32 *ptr = NULL;
+
+    size = size ? size : m_Size;
+    EXPECT_EQ((size & (sizeof(HSAuint32) - 1)), 0) << "Not word aligned. Call Fill(unsigned char)";
+    ASSERT_TRUE(size + offset <= m_Size) << "Buffer Overflow" << std::endl;
+
+    if (m_pUser != NULL)
+        ptr = reinterpret_cast<HSAuint32 *>(reinterpret_cast<char *>(m_pUser) + offset);
+
+    ASSERT_NOTNULL(ptr);
+
+    for (i = 0; i < size / sizeof(HSAuint32); i++)
+        ptr[i] = value;
+}
